@@ -75,12 +75,12 @@ fi
 #
 #
 BUILDDIR=$(mktemp -d --suffix=-build --tmpdir=.)
-KEY_ID="CFDDC96F12B1915C"
+KEY_ID="328D742D8807A435"
 
 # Try to make gpg cache the signing key, so we can leave the process
 # to run and sign.
-rm -f CHANGES.gpg
-gpg -s -u $KEY_ID CHANGES
+rm -f CMakeLists.txt.gpg
+gpg -s -u $KEY_ID CMakeLists.txt
 
 ### Build with default compiler
 #
