@@ -15,6 +15,7 @@
 #include "PackageModel.h"
 #include "ui_page_netinst.h"
 
+#include "GlobalStorage.h"
 #include "JobQueue.h"
 
 #include "network/Manager.h"
@@ -62,4 +63,19 @@ void
 NetInstallPage::onActivate()
 {
     ui->groupswidget->setFocus();
+
+    // The netinstallSelect global storage value can be used to make additional items selected by default
+    Calamares::GlobalStorage* gs = Calamares::JobQueue::instance()->globalStorage();
+    const QStringList selectNames = gs->value( "netinstallSelect" ).toStringList();
+    if ( !selectNames.isEmpty() )
+    {
+        m_config->model()->setSelections( selectNames );
+    }
+
+    // If NetInstallAdd is found in global storage, add those items to the tree
+    const QVariantList groups = gs->value( "netinstallAdd" ).toList();
+    if ( !groups.isEmpty() )
+    {
+        m_config->model()->appendModelData( groups );
+    }
 }
