@@ -42,7 +42,6 @@
 #include "widgets/PrettyRadioButton.h"
 
 #include <kpmcore/core/device.h>
-#include <kpmcore/core/partition.h>
 #ifdef WITH_KPMCORE4API
 #include <kpmcore/core/softwareraid.h>
 #endif
@@ -90,7 +89,6 @@ ChoicePage::ChoicePage( Config* config, QWidget* parent )
 
     auto gs = Calamares::JobQueue::instance()->globalStorage();
 
-    m_requiredPartitionTableType = gs->value( "requiredPartitionTableType" ).toStringList();
     m_enableEncryptionWidget = gs->value( "enableLuksAutomatedPartitioning" ).toBool();
 
     // Set up drives combo
@@ -178,10 +176,14 @@ ChoicePage::init( PartitionCoreModule* core )
 
 
     // We need to do this because a PCM revert invalidates the deviceModel.
-    connect( core, &PartitionCoreModule::reverted, this, [=] {
-        setModelToComboBox( m_drivesCombo, core->deviceModel() );
-        m_drivesCombo->setCurrentIndex( m_lastSelectedDeviceIndex );
-    } );
+    connect( core,
+             &PartitionCoreModule::reverted,
+             this,
+             [ = ]
+             {
+                 setModelToComboBox( m_drivesCombo, core->deviceModel() );
+                 m_drivesCombo->setCurrentIndex( m_lastSelectedDeviceIndex );
+             } );
     setModelToComboBox( m_drivesCombo, core->deviceModel() );
 
     connect( m_drivesCombo, qOverload< int >( &QComboBox::currentIndexChanged ), this, &ChoicePage::applyDeviceChoice );
@@ -305,26 +307,30 @@ ChoicePage::setupChoices()
 #else
     auto buttonSignal = &QButtonGroup::idToggled;
 #endif
-    connect( m_grp, buttonSignal, this, [this]( int id, bool checked ) {
-        if ( checked )  // An action was picked.
-        {
-            m_config->setInstallChoice( id );
-            updateNextEnabled();
+    connect( m_grp,
+             buttonSignal,
+             this,
+             [ this ]( int id, bool checked )
+             {
+                 if ( checked )  // An action was picked.
+                 {
+                     m_config->setInstallChoice( id );
+                     updateNextEnabled();
 
-            Q_EMIT actionChosen();
-        }
-        else  // An action was unpicked, either on its own or because of another selection.
-        {
-            if ( m_grp->checkedButton() == nullptr )  // If no other action is chosen, we must
-            {
-                // set m_choice to NoChoice and reset previews.
-                m_config->setInstallChoice( InstallChoice::NoChoice );
-                updateNextEnabled();
+                     Q_EMIT actionChosen();
+                 }
+                 else  // An action was unpicked, either on its own or because of another selection.
+                 {
+                     if ( m_grp->checkedButton() == nullptr )  // If no other action is chosen, we must
+                     {
+                         // set m_choice to NoChoice and reset previews.
+                         m_config->setInstallChoice( InstallChoice::NoChoice );
+                         updateNextEnabled();
 
-                Q_EMIT actionChosen();
-            }
-        }
-    } );
+                         Q_EMIT actionChosen();
+                     }
+                 }
+             } );
 
     m_rightLayout->setStretchFactor( m_itemsLayout, 1 );
     m_rightLayout->setStretchFactor( m_previewBeforeFrame, 0 );
@@ -403,11 +409,13 @@ ChoicePage::applyDeviceChoice()
     if ( m_core->isDirty() )
     {
         ScanningDialog::run(
-            QtConcurrent::run( [=] {
-                QMutexLocker locker( &m_coreMutex );
-                m_core->revertAllDevices();
-            } ),
-            [this] { continueApplyDeviceChoice(); },
+            QtConcurrent::run(
+                [ = ]
+                {
+                    QMutexLocker locker( &m_coreMutex );
+                    m_core->revertAllDevices();
+                } ),
+            [ this ] { continueApplyDeviceChoice(); },
             this );
     }
     else
@@ -495,11 +503,14 @@ ChoicePage::applyActionChoice( InstallChoice choice )
         if ( m_core->isDirty() )
         {
             ScanningDialog::run(
-                QtConcurrent::run( [=] {
-                    QMutexLocker locker( &m_coreMutex );
-                    m_core->revertDevice( selectedDevice() );
-                } ),
-                [=] {
+                QtConcurrent::run(
+                    [ = ]
+                    {
+                        QMutexLocker locker( &m_coreMutex );
+                        m_core->revertDevice( selectedDevice() );
+                    } ),
+                [ = ]
+                {
                     PartitionActions::doAutopartition( m_core, selectedDevice(), options );
                     Q_EMIT deviceChosen();
                 },
@@ -516,10 +527,12 @@ ChoicePage::applyActionChoice( InstallChoice choice )
         if ( m_core->isDirty() )
         {
             ScanningDialog::run(
-                QtConcurrent::run( [=] {
-                    QMutexLocker locker( &m_coreMutex );
-                    m_core->revertDevice( selectedDevice() );
-                } ),
+                QtConcurrent::run(
+                    [ = ]
+                    {
+                        QMutexLocker locker( &m_coreMutex );
+                        m_core->revertDevice( selectedDevice() );
+                    } ),
                 [] {},
                 this );
         }
@@ -534,11 +547,14 @@ ChoicePage::applyActionChoice( InstallChoice choice )
         if ( m_core->isDirty() )
         {
             ScanningDialog::run(
-                QtConcurrent::run( [=] {
-                    QMutexLocker locker( &m_coreMutex );
-                    m_core->revertDevice( selectedDevice() );
-                } ),
-                [this] {
+                QtConcurrent::run(
+                    [ = ]
+                    {
+                        QMutexLocker locker( &m_coreMutex );
+                        m_core->revertDevice( selectedDevice() );
+                    } ),
+                [ this ]
+                {
                     // We need to reupdate after reverting because the splitter widget is
                     // not a true view.
                     updateActionChoicePreview( m_config->installChoice() );
@@ -774,7 +790,8 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
 
     ScanningDialog::run(
         QtConcurrent::run(
-            [this, current, homePartitionPath]( bool doReuseHomePartition ) {
+            [ this, current, homePartitionPath ]( bool doReuseHomePartition )
+            {
                 QMutexLocker locker( &m_coreMutex );
 
                 if ( m_core->isDirty() )
@@ -855,7 +872,8 @@ ChoicePage::doReplaceSelectedPartition( const QModelIndex& current )
                 }
             },
             m_reuseHomeCheckBox->isChecked() ),
-        [this, homePartitionPath] {
+        [ this, homePartitionPath ]
+        {
             m_reuseHomeCheckBox->setVisible( !homePartitionPath->isEmpty() );
             if ( !homePartitionPath->isEmpty() )
                 m_reuseHomeCheckBox->setText( tr( "Reuse %1 as home partition for %2." )
@@ -1008,7 +1026,8 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
         connect( m_afterPartitionSplitterWidget,
                  &PartitionSplitterWidget::partitionResized,
                  this,
-                 [this, sizeLabel]( const QString& path, qint64 size, qint64 sizeNext ) {
+                 [ this, sizeLabel ]( const QString& path, qint64 size, qint64 sizeNext )
+                 {
                      Q_UNUSED( path )
                      sizeLabel->setText(
                          tr( "%1 will be shrunk to %2MiB and a new "
@@ -1022,7 +1041,8 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
         m_previewAfterFrame->show();
         m_previewAfterLabel->show();
 
-        SelectionFilter filter = []( const QModelIndex& index ) {
+        SelectionFilter filter = []( const QModelIndex& index )
+        {
             return PartUtils::canBeResized(
                 static_cast< Partition* >( index.data( PartitionModel::PartitionPtrRole ).value< void* >() ),
                 Logger::Once() );
@@ -1071,17 +1091,22 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
             eraseBootloaderLabel->setText( tr( "Boot loader location:" ) );
 
             m_bootloaderComboBox = createBootloaderComboBox( eraseWidget );
-            connect( m_core->bootLoaderModel(), &QAbstractItemModel::modelReset, [this]() {
-                if ( !m_bootloaderComboBox.isNull() )
-                {
-                    Calamares::restoreSelectedBootLoader( *m_bootloaderComboBox, m_core->bootLoaderInstallPath() );
-                }
-            } );
+            connect( m_core->bootLoaderModel(),
+                     &QAbstractItemModel::modelReset,
+                     [ this ]()
+                     {
+                         if ( !m_bootloaderComboBox.isNull() )
+                         {
+                             Calamares::restoreSelectedBootLoader( *m_bootloaderComboBox,
+                                                                   m_core->bootLoaderInstallPath() );
+                         }
+                     } );
             connect(
                 m_core,
                 &PartitionCoreModule::deviceReverted,
                 this,
-                [this]( Device* dev ) {
+                [ this ]( Device* dev )
+                {
                     Q_UNUSED( dev )
                     if ( !m_bootloaderComboBox.isNull() )
                     {
@@ -1112,7 +1137,8 @@ ChoicePage::updateActionChoicePreview( InstallChoice choice )
         }
         else
         {
-            SelectionFilter filter = []( const QModelIndex& index ) {
+            SelectionFilter filter = []( const QModelIndex& index )
+            {
                 return PartUtils::canBeReplaced(
                     static_cast< Partition* >( index.data( PartitionModel::PartitionPtrRole ).value< void* >() ),
                     Logger::Once() );
@@ -1219,18 +1245,22 @@ ChoicePage::createBootloaderComboBox( QWidget* parent )
     comboForBootloader->setModel( m_core->bootLoaderModel() );
 
     // When the chosen bootloader device changes, we update the choice in the PCM
-    connect( comboForBootloader, QOverload< int >::of( &QComboBox::currentIndexChanged ), this, [this]( int newIndex ) {
-        QComboBox* bootloaderCombo = qobject_cast< QComboBox* >( sender() );
-        if ( bootloaderCombo )
-        {
-            QVariant var = bootloaderCombo->itemData( newIndex, BootLoaderModel::BootLoaderPathRole );
-            if ( !var.isValid() )
-            {
-                return;
-            }
-            m_core->setBootLoaderInstallPath( var.toString() );
-        }
-    } );
+    connect( comboForBootloader,
+             QOverload< int >::of( &QComboBox::currentIndexChanged ),
+             this,
+             [ this ]( int newIndex )
+             {
+                 QComboBox* bootloaderCombo = qobject_cast< QComboBox* >( sender() );
+                 if ( bootloaderCombo )
+                 {
+                     QVariant var = bootloaderCombo->itemData( newIndex, BootLoaderModel::BootLoaderPathRole );
+                     if ( !var.isValid() )
+                     {
+                         return;
+                     }
+                     m_core->setBootLoaderInstallPath( var.toString() );
+                 }
+             } );
 
     return comboForBootloader;
 }
@@ -1252,6 +1282,28 @@ operator<<( QDebug& s, PartitionIterator& it )
     return s;
 }
 
+QString
+describePartitionTypes( const QStringList& types )
+{
+    if ( types.empty() )
+    {
+        return QCoreApplication::translate(
+            ChoicePage::staticMetaObject.className(), "any", "any partition-table type" );
+    }
+    if ( types.size() == 1 )
+    {
+        return types.first();
+    }
+    if ( types.size() == 2 )
+    {
+        return QCoreApplication::translate(
+                   ChoicePage::staticMetaObject.className(), "%1 or %2", "partition-table types" )
+            .arg( types.at( 0 ), types.at( 1 ) );
+    }
+    // More than two, rather unlikely
+    return types.join( ", " );
+}
+
 /**
  * @brief ChoicePage::setupActions happens every time a new Device* is selected in the
  *      device picker. Sets up the text and visibility of the partitioning actions based
@@ -1305,8 +1357,7 @@ ChoicePage::setupActions()
     if ( currentDevice->partitionTable() )
     {
         tableType = currentDevice->partitionTable()->type();
-        matchTableType = m_requiredPartitionTableType.size() == 0
-            || m_requiredPartitionTableType.contains( PartitionTable::tableTypeToName( tableType ) );
+        matchTableType = m_config->acceptPartitionTableType( tableType );
     }
 
     for ( auto it = PartitionIterator::begin( currentDevice ); it != PartitionIterator::end( currentDevice ); ++it )
@@ -1430,7 +1481,7 @@ ChoicePage::setupActions()
     }
 
 #ifdef DEBUG_PARTITION_UNSAFE
-#ifdef DEBUG_PARTITION_LAME
+#ifdef DEBUG_PARTITION_BAIL_OUT
     // If things can't be broken, allow all the buttons
     atLeastOneCanBeReplaced = true;
     atLeastOneCanBeResized = true;
@@ -1487,11 +1538,11 @@ ChoicePage::setupActions()
                                      "but the partition table <strong>%1</strong> is different from the "
                                      "needed <strong>%2</strong>.<br/>" )
                                      .arg( PartitionTable::tableTypeToName( tableType ) )
-                                     .arg( m_requiredPartitionTableType.join( " or " ) ) );
+                                     .arg( describePartitionTypes( m_config->partitionTableTypes() ) ) );
         m_messageLabel->show();
 
         cWarning() << "Partition table" << PartitionTable::tableTypeToName( tableType )
-                   << "does not match the requirement " << m_requiredPartitionTableType.join( " or " )
+                   << "does not match the requirement " << m_config->partitionTableTypes().join( ',' )
                    << ", ENABLING erase feature and DISABLING alongside, replace and manual features.";
         m_eraseButton->show();
         m_alongsideButton->hide();
@@ -1642,7 +1693,8 @@ ChoicePage::updateSwapChoicesTr()
             }
             else
             {
-                cWarning() << "Box item" << index << m_eraseSwapChoiceComboBox->itemText( index ) << "has non-integer role.";
+                cWarning() << "Box item" << index << m_eraseSwapChoiceComboBox->itemText( index )
+                           << "has non-integer role.";
             }
             break;
         case SwapChoice::ReuseSwap:
